type variable = string;;

type 'a environment =
    (variable * 'a) list;;

exception Unbound_variable of variable;;
exception Incompatible_environments of variable;;

let empty =
  [];;

let rec unbind searched_for_variable environment =
  match environment with
    | [] ->
      []
    | (a_variable, a_value) :: rest ->
      if a_variable = searched_for_variable then
        rest
      else
        (a_variable, a_value) :: (unbind searched_for_variable rest);;

let bind variable_to_bind value_to_bind environment =
  (variable_to_bind, value_to_bind) :: (unbind variable_to_bind environment);;

let rec bind_many bindings environment =
  match bindings with
    | [] ->
      environment
    | (variable, value) :: rest ->
      bind variable value (bind_many rest environment);;

let rec lookup searched_for_variable environment =
  match environment with
    | [] ->
      raise (Unbound_variable searched_for_variable)
    | (a_variable, a_value) :: rest ->
      if a_variable = searched_for_variable then
        a_value
      else
        lookup searched_for_variable rest;;

let rec is_bound searched_for_variable environment =
  match environment with
    | [] ->
      false
    | (a_variable, _) :: rest ->
      if a_variable = searched_for_variable then
        true
      else
        is_bound searched_for_variable rest;;

let merge first_environment second_environment =
  bind_many first_environment second_environment;;

let merge_disjoint first_environment second_environment =
  List.iter
    (fun (a_variable, _) ->
      if is_bound a_variable second_environment then
        raise (Incompatible_environments a_variable))
    first_environment;
  List.append first_environment second_environment;;

let pairs_of_environment e =
  e;;

let rec environment_of_pairs pairs =
  match pairs with
    | [] ->
      []
    | (variable, value) :: rest ->
      bind variable value (environment_of_pairs rest);;

let rec extend external_environment internal_environment =
  match internal_environment with
    | [] ->
      external_environment
    | (variable, value) :: rest ->
      bind variable value (extend external_environment rest);;
        
