open Domains;;

let binary_operator operator_name value1 value2 =
  match (operator_name, value1, value2) with
    | ("=", VInt i1, VInt i2) ->
      VBool(i1 = i2)
    | ("<", VInt i1, VInt i2) ->
      VBool(i1 < i2)
    | ("<=", VInt i1, VInt i2) ->
      VBool(i1 <= i2) 
    | (">", VInt i1, VInt i2) ->
      VBool(i1 > i2)
    | (">=", VInt i1, VInt i2) ->
      VBool(i1 >= i2)
    | ("+", VInt i1, VInt i2) ->
      VInt(i1 + i2)
    | ("-", VInt i1, VInt i2) ->
      VInt(i1 - i2)
    | ("*", VInt i1, VInt i2) ->
      VInt(i1 * i2)
    | ("/", VInt i1, VInt i2) ->
      VInt(i1 / i2)
    | ("&&", VBool b1, VBool b2) ->
      VBool(b1 && b2)
    | ("||", VBool b1, VBool b2) ->
      VBool(b1 || b2)
    | (",", left_value, right_value) ->
      VPair(left_value, right_value)
    | _ ->
      Printf.printf "%s\n" operator_name;
      failwith "error in primitive call";;

let rec expression_eval expression rho gamma =
  match expression with
    | EValue c ->
      c
    | EVariable x ->
      Environment.lookup x (Environment.extend gamma rho)
    | ELambda(x, _, e) ->
      VClosure(rho, x, e)
    | EApply(e1, e2) ->
      let VClosure(rho_c, x_c, e_c) =
        expression_eval e1 rho gamma
      and c = expression_eval e2 rho gamma
      in expression_eval e_c (Environment.bind x_c c rho_c) gamma
    | EBinaryPrimitive(e1, name, e2) ->
      (binary_operator name)
        (expression_eval e1 rho gamma)
        (expression_eval e2 rho gamma)
    | EIf(e1, e2, e3) ->
      if (expression_eval e1 rho gamma) = (VBool true) then
        expression_eval e2 rho gamma
      else
        expression_eval e3 rho gamma
    | ELet(x, _, e1, e2) ->
      let c = expression_eval e1 rho gamma
      in expression_eval e2 (Environment.bind x c rho) gamma;;

let toplevel_form_eval toplevel_form gamma =
  match toplevel_form with
    | TDefine(x, e) ->
      Environment.bind x (expression_eval e Environment.empty gamma) gamma
    | TDeclare(_, _) ->
      gamma;;

let rec program_eval program gamma =
  match program with
    | PExpression e ->
      expression_eval e Environment.empty gamma
    | PNonTrivial(t, p) ->
      program_eval p (toplevel_form_eval t gamma);;
