{
open Trivialml_parser;;
}

rule lexer = parse
[' ' '\t' '\n']+         {lexer lexbuf}
|"//"[^'\n']*'\n'         {lexer lexbuf}
|"()"                    {UnitObject}
|"("                     {OpenParenthesis}
|")"                     {CloseParenthesis}
|"+"                     {Plus}
|"-"                     {Minus}
|"*"                     {Times}
|"/"                     {Divided}
|"not"                   {Not}
|"@"                     {At}

|"ref"                   {Ref}
|":="                    {Set}
|"!"                     {Get}

|"zero?"                 {IsZero}
|"successor"             {Successor}
|"predecessor"           {Predecessor}

|"int"                   {IntType}
|"bool"                  {BoolType}
|"unit"                  {UnitType}
|"if"                    {If}
|"then"                  {Then}
|"else"                  {Else}
|"&&"                    {LogicAnd}
|"||"                    {LogicOr}
|"and"                   {And}
|"left"                  {Left}
|"right"                 {Right}
|"let"                   {Let}
|"val"                   {Val}
|"in"                    {In}
|"rec"                   {Rec}
|"lambda"                   {Lambda}
|"match"                 {Match}
|"with"                  {With}
|"|"                     {Pipe}
|"->"                    {Arrow}
|"."                     {Dot}
|";;"                    {Terminator}
|"="                     {Equals}
|","                     {Comma}
|"True"                  {True}
|"False"                 {False}
|"_"                     {Underscore}
|":"                     {Colon}
|('-'|'+')?['0'-'9']+ as s {Int (int_of_string s)}
|[^ ',' '.' ' ' '\t' '\n' '\r' '(' ')' '=' '/' ';']+ as s {Identifier s}
(*
|((['A'-'Z']([^ ',' '.' ' ' '\t' '\n' '\r' '(' ')' '=' '/']+))) as s {Variable s}
|((['a'-'z']([^ ',' '.' ' ' '\t' '\n' '\r' '(' ')' '=' '/']+))) as s {NonVariable s}
|([' ' '\t' '\n']+) as q   {Printf.printf "About to fail: the token is \"%s\"\n" q; assert false}
*)
| eof                    {EOF}

{
let parse_stdin () =
  let lexbuf = Lexing.from_channel stdin in
(*   try *)
  Printf.printf "Parsing from stdin...\n"; flush_all ();
  let result = main lexer lexbuf in
  Printf.printf "Parsed with success\n";
  result
(*   with _ -> *)
(*     print_string "Parsing failed with an exception\n" *)
;;

(*let _ = main ();;*)
}
