%{
open Domains;;
%}

%left LOWESTPRECEDENCE

/* Rec must have lowest precedence among the tokens occurring within definitions: */
%left Rec

%token Val Let In And Rec Equals
%left Let In Equals
%left And
%token Left Right
%right Arrow
%token Arrow Lambda Dot

%token OpenParenthesis
%token CloseParenthesis
%token LogicOr
%token LogicAnd
%token Not
%token Match With
%token Pipe
%token Terminator
%token Comma Semicolon Colon
%token Underscore
%token True False Int UnitObject
%token Identifier
%type <string> Identifier
%token Plus Minus Times Divided

%token Ref Set Get

%token IsZero Successor Predecessor

%token IntType BoolType UnitType

%left Plus Minus
%left Times Divided
%right Comma

%token If Then Else

%nonassoc Not

%start main

%type <Domains.program> main

%type <string> Identifier
%type <int> Int

%token At
%left At

%token EOF

%left NEXTHIGHESTPRECEDENCE
%left HIGHESTPRECEDENCE

%%

main :
| program EOF { $1 }
;

program :
| expression            { PExpression $1 }
| toplevel_form program { PNonTrivial($1, $2) }
;

toplevel_form :
| Let Identifier Equals expression Terminator { TDefine($2, $4) }
| Val Identifier Colon type_ Terminator { TDeclare($2, $4) }
;

/* This is quite ugly, but I didn't find any simple way of making
   application-by-justapposition left-associative and with lowest
   precedence than let..in.  Yes, these rules must come before the
   rules for expression, so that reduce/reduce conflicts are solved
   the way we want. */
possibly_trivial_application:
| possibly_trivial_application expression %prec HIGHESTPRECEDENCE { EApply($1, $2) }
| expression                                                      { $1 }
;

expression :
| value                                       { EValue($1) }
| Identifier                                  { EVariable $1 }

| Lambda Identifier Colon type_ Dot expression { ELambda($2, $4, $6) }

| possibly_trivial_application expression    { EApply($1, $2) }

| If expression Then expression Else expression { EIf($2, $4, $6) }

| expression Plus expression                  { EBinaryPrimitive($1, "+", $3) }
| expression Minus expression                 { EBinaryPrimitive($1, "-", $3) }
| expression Times expression                 { EBinaryPrimitive($1, "*", $3) }
| expression Divided expression               { EBinaryPrimitive($1, "/", $3) }
| expression Equals expression                { EBinaryPrimitive($1, "=", $3) }

| expression LogicAnd expression              { EBinaryPrimitive($1, "&&", $3) }
| expression LogicOr expression               { EBinaryPrimitive($1, "||", $3) }

| expression Comma expression                 { EBinaryPrimitive($1, ",", $3) }

| Left expression                             { EUnaryPrimitive("left", $2) }
| Right expression                            { EUnaryPrimitive("right", $2) }
| Not expression                              { EUnaryPrimitive("not", $2) }

| OpenParenthesis expression CloseParenthesis { $2 }
| Let Identifier Colon type_ Equals expression In expression
                                              { ELet($2, $4, $6, $8) }
;

type_ :
| IntType { TInt }
| BoolType { TBool }
| type_ Arrow type_ { TFunction($1, $3) }
| type_ Times type_ { TPair($1, $3) }
| OpenParenthesis type_ CloseParenthesis { $2 }
;

value :
| Int        { VInt $1 }
| True       { VBool true }
| False      { VBool false }
;

%%

(* This code is executed at initialization time. *)
